/******************************************************
 *                         MergeKW.h
 *
 * Project : MergeKW                       Ver : 1.0
 * Programmer : Michael Taylor        Date : 8/16/98
 * Language : VC 5.0                Target : Win32
 * Purpose : This file provides the interface to the
 *           boiler plate code for MFC and the DLL.
 *****************************************************/
//@DOC

#if !defined(MERGEKW_H_INCLUDED)
#define MERGEKW_H_INCLUDED
#pragma once

#ifndef __AFXWIN_H__
	#error include 'stdafx.h' before including this file for PCH
#endif

#include "MergeKW.rh"		// main symbols
#include "commands.h"
#include <stdarg.h>

#include <ObjModel\addguid.h>
#include <ObjModel\appguid.h>
#include <ObjModel\bldguid.h>
#include <ObjModel\textguid.h>
#include <ObjModel\dbgguid.h>
#include <atlbase.h>

//Maximum size of a description
#define MAX_SIZE 100

//Extern this so we can get the path later
extern CComModule _Module;

//Merges all selected files
bool MergeFiles ( bool bGUI = true );

/*@ENUM
 *   Types of messages
 */
enum MSG_TYPE
{
   INFO_MSG = 0,      //@EMEM A normal message
   WARNING_MSG = 1,   //@EMEM A warning message
   ERROR_MSG = 2      //@EMEM An error message
};

/*@FUNC
 *   Displays a message to the user.
 *@COMM
 *   This function supports several different
 * types of messages.  Depending on the <p Type>
 * parameter, it displays an appropriate icon along
 * with the message.  This function also supports 
 * format specifiers in the resource strings.
 *@DEVNOTE
 *   Note that due to a problem with CString's format
 * method, we have to use the standard C runtime printf
 * function.  Thus strings are limited to 1024 characters.
 *@RDESC
 *   None.
 *@XREF
 *   <t MSG_TYPE>
 *@AUTHOR
 *   Michael Taylor
 *@HISTORY
 *   8/22/98 Adapted from existing version for external use
 */
inline void _cdecl DisplayMsg (
                      MSG_TYPE Type,   //@PARM [IN] Type of message
                      UINT uiID,      //@PARM [IN] ID of message to display
                      ...
                         )
{
   CString szMsg = TEXT("");
   CString szMsg2 = TEXT("");
   DWORD dwFlags;

   va_list args;
   va_start(args, uiID);

   //We can't use Format directly as we already have the
   //va_list, so call the FormatV method instead   
   szMsg2.LoadString(uiID);
   if (szMsg2.IsEmpty())
      szMsg.LoadString(IDS_GENERAL_ERROR);
   else		
	{
#if (_MFC_VER >= 0x0600)
      szMsg.FormatV(szMsg2, args);
#else		
		//Permit messages up to 100 characters
		_vstprintf(szMsg.GetBufferSetLength(100), szMsg2, args);
		szMsg.ReleaseBuffer();
#endif
	};

   switch (Type)
   {
      case ERROR_MSG   : dwFlags = MB_ICONERROR; break;
      case WARNING_MSG : dwFlags = MB_ICONWARNING; break;
      default          : dwFlags = MB_ICONINFORMATION; break;
   };

   AfxMessageBox(szMsg, dwFlags);

   va_end(args);
};

/*@FUNC
 *   Gets the directory where the add-in resides at.
 *@RDESC
 *   The directory or an empty string on error.
 *@XREF
 *   <f GetAddInDir>
 *@AUTHOR
 *   Michael Taylor
 *@HISTORY
 *   8/22/98  Original Version
 */
inline CString GetMSDevDir ( void )
{
   TCHAR Buffer[MAX_PATH + 1] = TEXT("");
   CString szPath;

   //Get the name of the current process, this will be MSDEV
   GetModuleFileName(NULL, Buffer, sizeof(Buffer));
   szPath = Buffer;

   //Strip off the filename
   int iPos = szPath.ReverseFind(TEXT('\\'));
   if (iPos != -1)
      szPath = szPath.Left(iPos);
   return(szPath);
};

/*@FUNC
 *   Gets the directory where the add-in is installed.
 *@RDESC
 *   The directory, or an empty string.
 *@AUTHOR
 *   Michael Taylor 
 *@HISTORY
 *   9/9/98  Original version
 */   
inline CString GetAddInDir ( void )
{
   TCHAR Buffer[MAX_PATH] = TEXT("");
   CString szPath;

   //Get the filename and strip off the file
   GetModuleFileName(_Module.GetModuleInstance(), Buffer, sizeof(Buffer));
   szPath = Buffer;
   int iPos = szPath.ReverseFind(TEXT('\\'));
   if (iPos != -1)
      szPath = szPath.Left(iPos);
      
   return(szPath);
};

/*@FUNC
 *   Appends the specified file, <p szFileName> to the
 * given file, <p File>.
 *@COMM
 *   This function will generate an error if the append
 * fails.
 *@RDESC
 *   None.
 *@AUTHOR
 *   Michael Taylor
 *@HISTORY
 *   8/22/98  Original Version
 */
inline void MergeFile ( 
               CString szFileName,     //@PARM [IN] File to add
               CStdioFile& File        //@PARM [INOUT] Target file
                      )
{
   CStdioFile SrcFile;
   CString szLine;

   //Attempt to open the source file
   if (!SrcFile.Open(szFileName, CFile::modeRead | CFile::shareDenyWrite |
                     CFile::typeText))
   {
#if defined(_UNICODE) || defined(UNICODE)
      DisplayMsg(ERROR_MSG, IDS_FMTOPEN_FAILED, szFileName);
#else
      USES_CONVERSION;
      DisplayMsg(ERROR_MSG, IDS_FMTOPEN_FAILED, A2W(szFileName));
#endif
      return;
   };

   //Write out a header
   szLine.Format(IDS_FMTFILE_HEADER, szFileName);
   File.WriteString(szLine);

   //While more data exists
   while (SrcFile.ReadString(szLine))
   {
      szLine += TEXT('\n');
      File.WriteString(szLine);      
   };

   //Clean up
   SrcFile.Close();
};

/*@FUNC
 *   Builds a string consisting of the given path and
 * the given filename.
 *@MSYNTAX CString BuildPath ( CString szPath, CString szFileName );
 *@MSYNTAX CString BuildPath ( CString szPath, UINT uiFileName );
 *@MSYNTAX CString BuildPath ( UINT uiPath, CString szFileName );
 *@MSYNTAX CString BuildPath ( UINT uiPath, UINT uiFileName );
 *@COMM
 *   This function can handle paths that end in slashes.
 * It can handle either strings as parameters or
 * resource IDs as parameters.
 *@DEVNOTE
 *   Just by chance, this function will also probably work with
 * registry keys.
 *@RDESC
 *   The new string or the empty string on error.
 *@AUTHOR
 *   Michael Taylor
 *@HISTORY
 *   8/25/98 Original version
 */
inline CString BuildPath (
                  CString szPath,      //@PARM [IN] Path 
                  CString szFileName   //@PARM [IN] Filename
                         )
{
   //Check for empty strings
   if (szPath.IsEmpty() || szFileName.IsEmpty())
      return(TEXT(""));

   //Check for a slash
   if ((szPath.Compare(TEXT("\\")) == 0) ||
       (szPath.Compare(TEXT("/")) == 0))
      return(szPath + szFileName);
   else
      return(szPath + TEXT("\\") + szFileName); 
};
inline CString BuildPath (
                  CString szPath,      
                  UINT uiFileName      //@PARM [IN] Resource ID of filename
                         )
{
   //Get the actual filename
   CString szFileName = L"";
   szFileName.LoadString(uiFileName);

   //Call the real function
   return(BuildPath(szPath, szFileName));
};
inline CString BuildPath (
                  UINT uiPath,      //@PARM [IN] Resource ID of path
                  CString szFileName
                         )
{
   //Get the actual path
   CString szPath = L"";
   szPath.LoadString(uiPath);
   
   //Call the real function
   return(BuildPath(szPath, szFileName));     
};
inline CString BuildPath ( UINT uiPath, UINT uiFileName )
{
   //Get the actual path
   CString szPath = L"";
   szPath.LoadString(uiPath);

   //Get the actual filename
   CString szFileName = L"";
   szFileName.LoadString(uiFileName);
   
   //Call the real function
   return(BuildPath(szPath, szFileName));     
};

/*@FUNC
 *   Deletes any existing registry entries for the add-in
 * such that we start with a clean key.
 *@RDESC
 *   Returns TRUE if successful, or FALSE otherwise.
 *@AUTHOR
 *   Michael Taylor 
 *@HISTORY
 *   8/27/98  Original version
 */   
inline bool CleanReg ( void )
{
   CString szBaseKey;
   CRegKey BaseKey;

   //Open the base key
   if (BaseKey.Open(HKEY_CURRENT_USER, BASE_KEY) == ERROR_SUCCESS)
   {
      //Delete everything
      BaseKey.RecurseDeleteKey(FILES_KEY);
      BaseKey.Close();
   };
   return(true);
};

//{{AFX_INSERT_LOCATION}}
// Microsoft Developer Studio will insert additional declarations immediately before the previous line.
//}}AFX_INSERT_LOCATION}}
#endif  // Sentry
